#include <iostream>
#include <omp.h>
#include "timing.h"

using namespace std;

int main() {

  const int Ni=500, Nj=500, Nk=500;
  const int jblock=Nj;
  const int Ni2=Ni+2, Nj2=Nj+2, Nk2=Nk+2;
  int       iter=10, threads=0;
  const double f=1./6.;
//  double x[Ni+2][Nj+2][Nk+2], y[Ni+1][Nj+1][Nk+1];
  double wct_start=0, wct_end=0;
  double cput_start=0, cput_end=0;

  double *phi[2]; 
  phi[0] = new double[Ni2*Nj2*Nk2];  // x
  phi[1] = new double[Ni2*Nj2*Nk2];  // y

  // initialize x
  #pragma omp parallel
  {
  #pragma omp for
  for (int i=0; i<Ni2; i++)
    for (int j=0; j<Nj2; j++) {
      int ofs = i*Nj2*Nj2 + j*Nk2;
      for (int k=0; k<Nk2; k++) {
         phi[0][ofs+k] = double(i+j+k);
         phi[1][ofs+k] = 0.0;
      }
    }

  // time steps
  int t0=0, t1=1; 


  // start clock
  #pragma omp master
  timing(&wct_start, &cput_start);
  
  for (int n=0; n<iter; n++) {
      // 
      for (int jb=1; jb<=Nj; jb+=jblock) {

          // do work (5 add and one multiplication)
          #pragma omp for
          for (int i=1; i<=Ni; i++)
             for (int j=jb; j<=(jb+jblock-1); j++) {
                int ofs = i*Nj2*Nj2 + j*Nk2; 
                for (int k=1; k<=Nk; k++)
                   phi[t1][ofs+k] = f*( phi[t0][ofs+k-1] + phi[t0][ofs+k+1]
                                       +phi[t0][ofs+k-Nk2] + phi[t0][ofs+k+Nk2]
                                       +phi[t0][ofs+k-Nj2*Nj2] + phi[t0][ofs+k+Nj2*Nj2] );
             }
      }
      int t=t0;
      t0=t1; t1=t; // swap
  }
  // stop clock and print time
  #pragma omp master
  timing(&wct_end, &cput_end);

  threads = omp_get_num_threads();
  } // end pragma omp parallel


  double wct   = wct_end-wct_start;
  double niter = double(iter)*Ni*Nj*Nk;
  double perf  = niter*6/wct/1e9;
  double bw    = niter*48/wct/1e9; 

  cout << "Threads:     " << threads << endl;
  cout << "Wallclock:   " << wct << endl;
  cout << "Iterations:  " << niter << endl;
  cout << "Performance: " << perf << endl;
  cout << "Bandwidth:   " << bw << endl; 

}

